#!/usr/bin/env python


__author__ = "Bartlomiej Tomasz Urbaniec"
__email__ = "bartlomiej.tomasz.urbaniec@cern.ch"
__copyright__ = "(C) 2011 European Organization for Nuclear Research, Geneva, Switzerland"
__version__ = "1.1"
__date__ = "$Date: 2012-02-03 16:28:41 +0100 (Fri, 03 Feb 2012) $"
__description__ = "Plugin for FlexExtractor to handle databases using ODBC"
__status__ = "Development" 
 



# =============================================
#  FUNCTION RESPONSIBLE FOR READING DATA FROM A CERTAIN
#  INPUT. THIS FUNCTION IS CALLED BY FlexExtractor.
# =============================================
def getData(pFile, pSourceList):
    """
    This function reads data from an input source specified in the parameter "pFile". The structure
    defined in "pSourceList" is used to know what data to read from the input source. This function
    returns a tuple with four elements: code, header, data and message.

    Parameter(s):
       pFile - string containing the name of the input source to be read.     
       pSourceList - list containing the input structure that was specified in the XML config.
    
    Return(s):
       int - 0 (the input source was successfully read) or -1 (error found when reading the input source).
       list - contains a list with the headers of the input source (this list should be empty when the "autoselect"
              is False and filled-up when "autoselect" is True).                    
       list - contains the data that was read from the input source (this list will be empty if some error was found).
       list - contains a list of messages that were generated during the reading of the input source.             
    """
        
    try:
        import pyodbc
    except:
        return -1, [], [], ["Error when importing module 'pyodbc'."]
    
    try:

        autoSelectList = []
        messageList = []        
        result = []
        sqlQuery = ""
        #establish db connection
        connection = pyodbc.connect(pFile)
        cursor = connection.cursor()
        
        for source in pSourceList:
            flag = False
            tempRows = []
            tmpHeader = []
            namesToRead = ""
            
            if source["name"] is None or source["name"] == "":
                sqlQuery = "Missing table name (source name)"
                raise pyodbc.ProgrammingError
            
            #list of columns for given table
            columnsList = []
            for column in cursor.columns(table = source["name"]):
                columnsList.append(column.column_name)            
            
            if source["autoselect"] is not None and source["autoselect"].upper()  == "TRUE":
                namesToRead = "*,"
                flag = True
                for column in columnsList:
                    tmpHeader.append(column)
            
            
            for d in source["data"]:
                if d["select"] not in columnsList:
                    sqlQuery = "Column '%s' doesn't exist in table '%s'" % (d["select"], source["name"])
                    raise pyodbc.ProgrammingError                    
                
                namesToRead += d["select"] + ","
                if flag:
                    tmpHeader.append(d["name"])
                
            #remove last comma - not needed
            namesToRead = namesToRead[0:len(namesToRead)-1] 
            sqlQuery = "Select " + namesToRead + " from " + source["name"]
            cursor.execute(sqlQuery)
            
            while True:
                row = cursor.fetchone()
                if not row:
                    break
                tempRows.append(list(row))
            autoSelectList.append(tmpHeader)
            result.append(tempRows)      
        connection.close()    
        return 0, autoSelectList, result, messageList     
    
    except MemoryError:
        return -1, [], [], ["Error when connecting to DB with '%s' since there is not enough memory available." % pFile]                    
    except pyodbc.ProgrammingError:
        return -1, [], [], ["Error when executing query: %s." % sqlQuery]
    except Exception, e:
        return -1, [], [], ["Error '%s' when connecting to DB with '%s'." % (e,pFile)]


    


# =============================================
#  FUNCTION RESPONSIBLE FOR WRITING DATA TO A CERTAIN
#  OUTPUT. THIS FUNCTION IS CALLED BY FlexExtractor.
# =============================================
def setData(pFile, pParameter, pDataList, pAppendFlag):
    """
    This function writes the data "pDataList" into a certain output specified in the parameter "pFile"
    and uses the parameter "pParameter" to provide additional information on how to do it. If "pAppendFlag"
    is True then the data will be appended in the end of the output, otherwise (if False) the previous
    data will be erased before writing the new data. This function returns a tuple with two elements:
    code and message.

    Parameter(s):
       pFile - string containing the name of the output where to write the data.    
       pParameter - string containing the parameter that was specified in the XML config.    
       pDataList - list containing the data to be written.
       pAppendFlag - boolean that specifies if the output is to be overwritten (True) or appended (False) with the new data.
    
    Return(s):
       int - 0 (the data was successfully written) or -1 (error found when writing the data).             
       list - contains a list of messages that were generated during the writing of the data.      
    """
    
    return 0, []
    

