#!/usr/bin/env python


__author__ = "Ricardo Fernandes"
__email__ = "ricardo.fernandes@cern.ch"
__copyright__ = "(C) 2011 European Organization for Nuclear Research, Geneva, Switzerland"
__version__ = "1.2"
__date__ = "$Date: 2012-03-07 11:20:07 +0100 (Wed, 07 Mar 2012) $"
__description__ = "Plugin for FlexExtractor to handle SPEC XML files"
__status__ = "Production" 
 



# =============================================
#  FUNCTION RESPONSIBLE FOR READING DATA FROM A CERTAIN
#  INPUT. THIS FUNCTION IS CALLED BY FlexExtractor.
# =============================================
def getData(pFile, pSourceList):
    """
    This function reads data from an input source specified in the parameter "pFile". The structure
    defined in "pSourceList" is used to know what data to read from the input source. This function
    returns a tuple with four elements: code, header, data and message.

    Parameter(s):
       pFile - string containing the name of the input source to be read.     
       pSourceList - list containing the input structure that was specified in the XML config.
    
    Return(s):
       int - 0 (the input source was successfully read) or -1 (error found when reading the input source).
       list - contains a list with the headers of the input source (this list should be empty when the "autoselect"
              is False and filled-up when "autoselect" is True).                    
       list - contains the data that was read from the input source (this list will be empty if some error was found).
       list - contains a list of messages that were generated during the reading of the input source.             
    """
    
    try:
        import os
    except:
        return -1, [], [], ["Error when importing module 'os'."]
    
    try:
        import re
    except:
        return -1, [], [], ["Error when importing module 're'."]
    
    try:
        from xml.dom import minidom
    except:
        return -1, [], [], ["Error when importing module 'minidom'."]
            
    try:
        from xpath.expr import xpath
    except:
        return -1, [], [], ["Error when importing module 'xpath'."]

    if not os.path.isfile(pFile):
        return -1, [], [], ["The input file '%s' does not exists." % pFile]    
    
      
    try:
        parser = minidom.parse(pFile)        
        
        autoSelectList = []
        messageList = []        
        result = []
        for source in pSourceList:
           
            # =============================================
            # GET DATA START IF RANGE IS NONE
            # =============================================
            if source["range"] is None:
                tmpHE = xpath.findvalue("/*:Workbook/*:Worksheet[@ss:Name='%s']/*:Table/@ss:HeaderEnd/text()" % source["name"], parser)
                if tmpHE is not None:
                    tmpHE = int(tmpHE) + 1
                    source["range"] = str(tmpHE) + ":"
            
            rowList = xpath.find("/*:Workbook/*:Worksheet[@ss:Name='%s']/*:Table/*:Row" % source["name"], parser)
            if rowList == []:
                autoSelectList.append([])                
                result.append([])
                messageList.append("The input file '%s' does not have a sheet '%s' or it is empty." % (pFile, source["name"]))
            else:
                
                sourceAutoSelect = source["autoselect"]
                flag = True
                if sourceAutoSelect is None:
                    autoSelectFlag = False
                    startHeader = xpath.findvalue("/*:Workbook/*:Worksheet[@ss:Name='%s']/*:Table/@ss:HeaderStart/text()" % source["name"], parser)
                    endHeader = xpath.findvalue("/*:Workbook/*:Worksheet[@ss:Name='%s']/*:Table/@ss:HeaderEnd/text()" % source["name"], parser)
                    if startHeader is None or endHeader is None:
                        autoSelectList.append([])
                        result.append([])
                        messageList.append("The sheet '%s' doesn't have the 'HeaderStart' or 'HeaderEnd' attributes defined." % source["name"])
                        continue   
                        #return -1, [], [], ["The sheet '%s' doesn't have the 'HeaderStart' or 'HeaderEnd' attributes defined." % source["name"]]
                    else:                        
                        startHeader = int(startHeader) - 1
                        endHeader = int(endHeader)                                                          
                else:
                    autoSelectFlag = True
                    if sourceAutoSelect.upper() == "TRUE":                               
                        startHeader = xpath.findvalue("/*:Workbook/*:Worksheet[@ss:Name='%s']/*:Table/@ss:HeaderStart/text()" % source["name"], parser)
                        endHeader = xpath.findvalue("/*:Workbook/*:Worksheet[@ss:Name='%s']/*:Table/@ss:HeaderEnd/text()" % source["name"], parser)
                        if startHeader is None or endHeader is None:
                            autoSelectList.append([])
                            result.append([])
                            messageList.append("The sheet '%s' doesn't have the 'HeaderStart' or 'HeaderEnd' attributes defined." % source["name"])
                            continue   
                            #return -1, [], [], ["The sheet '%s' doesn't have the 'HeaderStart' or 'HeaderEnd' attributes defined." % source["name"]]
                        else:                        
                            startHeader = int(startHeader) - 1
                            endHeader = int(endHeader)                        
                    else:
                        try:           
                            sourceAutoSelectList = sourceAutoSelect.split(":", 2)
                            if len(sourceAutoSelectList) == 1:
                                startHeader = int(sourceAutoSelectList[0]) - 1
                                endHeader = int(sourceAutoSelectList[0])
                            else:
                                startHeader = int(sourceAutoSelectList[0]) - 1
                                endHeader = int(sourceAutoSelectList[1])                                                        
                        except:
                            flag = False
                            autoSelectList.append([])
                            messageList.append("The sheet '%s' has an invalid autoselect '%s' (it should specify the start/end of the header rows)." % (source["name"], sourceAutoSelect))            
                
                
                tmpData0 = []
                if flag:
                    # =============================================
                    #  FILLING EMPTY ROW IF INDEX EXIST FOR ROW
                    # ============================================= 
                    newRowList = []
                    rowIterator = 1
                    for tmpRow in rowList:
                        rowIndex = xpath.findvalue("./@ss:Index/text()", tmpRow)
                        if rowIndex is not None and rowIterator <= int(rowIndex):
                            while rowIterator < int(rowIndex):  
                                rowElementTmp = parser.createElement("Row")
                                cellElementTmp = parser.createElement("Cell")
                                rowElementTmp.appendChild(cellElementTmp)
                                newRowList.append(rowElementTmp)
                                rowIterator += 1
                        newRowList.append(tmpRow)
                        rowIterator += 1
                    rowList = []
                    rowList = newRowList            
                           
                    # =============================================
                    #  GET HEADER
                    # =============================================                    
                    tmpHeaderList0 = []
                    
                    for row in rowList[startHeader:endHeader]:
                        cellList = xpath.find("./*:Cell", row)
                        tmpHeaderList1 = []               
                        for cell in cellList:
                            value = xpath.findvalue("./*:Data/text()", cell)
                            if value is None:
                                value = ""
                            mergeAcross = xpath.findvalue(".//@ss:MergeAcross/text()", cell)                        
                            if mergeAcross is None:
                                mergeAcross = 1
                            else:
                                mergeAcross = int(mergeAcross) + 1
                            for i in range(mergeAcross):
                                tmpHeaderList1.append(value)
                        tmpHeaderList0.append(tmpHeaderList1)
                    tmpAutoSelectList = []             
                    headerList = {}                
                    if tmpHeaderList0 != []:
                        for i in range(len(tmpHeaderList0[0])):
                            header = ""
                            for j in range(len(tmpHeaderList0)):
                                if header == "":
                                    header = "%s" % tmpHeaderList0[j][i]
                                else:
                                    if tmpHeaderList0[j][i] != "":
                                        header = "%s.%s" % (header, tmpHeaderList0[j][i])
                            headerList[header] = i
                            for data in source["data"]:
                                if data["regexpselect"] is not None and data["regexpselect"].upper() == "TRUE":
                                    try:
                                        if re.search(data["select"], header) is not None:
                                            data["select"] = header
                                    except:
                                        pass                                    
                            tmpAutoSelectList.append(header)
                    autoSelectList.append(tmpAutoSelectList)
                    

                    # =============================================
                    #  GET DATA
                    # =============================================
                    i = len(headerList)
                    for row in rowList:
                        rowStyleID = xpath.findvalue(".//@ss:StyleID/text()", row)                    
                        cellList = xpath.find("./*:Cell", row)                    
                        tmpData1 = [""] * i                  
                        j = 0                    
                        for cell in cellList:
                            cellStyleID = xpath.findvalue(".//@ss:StyleID/text()", cell)
                            if cellStyleID is None:
                                if rowStyleID is not None:
                                    cellStyleID = rowStyleID
                                else:
                                    cellStyleID = ""
                            value = xpath.findvalue("./*:Data/text()", cell)
                            if value is None:
                                value = ""
                            index = xpath.findvalue(".//@ss:Index/text()", cell)
                            value = cellStyleID + "&&&" + value
                            if index is None:
                                j += 1
                            else:  
                                j = int(index)
                            if j <= i:                          
                                tmpData1[j - 1] = value                            
                        if autoSelectFlag:
                            tmpData2 = tmpData1
                        else:
                            tmpData2 = []                    
                        for data in source["data"]:
                            if data["select"] is None:
                                tmpData2.append(None)
                            else:                                     
                                try:
                                    tmpData2.append(tmpData1[headerList[data["select"]]])
                                except:
                                    message = "The input file '%s' does not have a select '%s' in sheet '%s' or it is invalid." % (pFile, data["select"], source["name"])
                                    if message not in messageList:
                                        messageList.append(message)                                
                                    tmpData2.append(None)
                        tmpData0.append(tmpData2)                        
                result.append(tmpData0)
                
        return 0, autoSelectList, result, messageList     
    except MemoryError:
        return -1, [], [], ["Error when reading input file '%s' since there is not enough memory available." % pFile]                    
    except:
        return -1, [], [], ["Error when reading/parsing input file '%s'." % pFile]


    


# =============================================
#  FUNCTION RESPONSIBLE FOR WRITING DATA TO A CERTAIN
#  OUTPUT. THIS FUNCTION IS CALLED BY FlexExtractor.
# =============================================
def setData(pFile, pParameter, pDataList, pAppendFlag):
    """
    This function writes the data "pDataList" into a certain output specified in the parameter "pFile"
    and uses the parameter "pParameter" to provide additional information on how to do it. If "pAppendFlag"
    is True then the data will be appended in the end of the output, otherwise (if False) the previous
    data will be erased before writing the new data. This function returns a tuple with two elements:
    code and message.

    Parameter(s):
       pFile - string containing the name of the output where to write the data.    
       pParameter - string containing the parameter that was specified in the XML config.    
       pDataList - list containing the data to be written.
       pAppendFlag - boolean that specifies if the output is to be overwritten (True) or appended (False) with the new data.
    
    Return(s):
       int - 0 (the data was successfully written) or -1 (error found when writing the data).             
       list - contains a list of messages that were generated during the writing of the data.      
    """
    
    
    try:
        import sys
    except:
        return -1, ["Error when importing module 'sys'."]    
    
    try:
        import os
    except:
        return -1, ["Error when importing module 'os'."]        
    
    try:
        from xml.dom import minidom
    except:
        return -1, ["Error when importing module 'minidom'."]
            
    if not os.path.isfile(pFile):
        return -1, ["The output file '%s' does not exists." % pFile]
    
    if pParameter is None:
        separator = ","
    else:
        separator = pParameter    
       
    messageList = []
    try:  
        
        parser = minidom.parse(pFile)
        sheetList = parser.getElementsByTagName("Worksheet")
                
        if True or sys.platform[:4] == "java":
            inputMessages = parser.getElementsByTagName("InputMessage")
            for inputMessage in inputMessages:
                child = inputMessage.firstChild
                cdata = parser.createCDATASection(child._get_wholeText())
                inputMessage.removeChild(child)
                inputMessage.appendChild(cdata)
        
        
        if sheetList == []:
            return -1, ["The output file '%s' does not have sheets." % pFile]
        else:
            i = 0
            while i < len(pDataList):
                tmpData = pDataList[i].upper()                
                if tmpData.startswith("SHEET:"):
                    sheetName = pDataList[i][6:]
                    flag0 = False
                elif tmpData.startswith("SHEETDELETE:"):
                    sheetName = pDataList[i][12:]
                    flag0 = True
                else:
                    sheetName = ""                
                if sheetName != "":
                    flag1 = False
                    flag2 = False
                    for sheet in sheetList:
                        if sheetName == sheet.getAttribute("ss:Name"):
                            flag1 = True                            
                            tableList = sheet.getElementsByTagName("Table")
                            if tableList == []:
                                messageList.append("The sheet '%s' in output file '%s' does not have a table." % (sheetName, pFile))
                            else:
                                table = tableList[0]                                
                                if not pAppendFlag:
                                    j = int(table.getAttribute("ss:HeaderEnd"))
                                    while len(table.getElementsByTagName("Row")) > j:
                                        table.removeChild(table.getElementsByTagName("Row")[j])
                                for j in range(i + 1, len(pDataList)):
                                    tmpData = pDataList[j].upper()                                    
                                    if tmpData.startswith("SHEET:") or tmpData.startswith("SHEETDELETE:"):
                                        break
                                    else:                              
                                        rowElement = parser.createElement("Row")
                                        rowElement.setAttribute("ss:AutoFitHeight", "0")
                                        dataList = pDataList[j].split(separator)
                                        for data in dataList:
                                            cellElement = parser.createElement("Cell")
                                            #data = data.decode("utf-8")
                                            tmpData = data.split("&&&")
                                            if len(tmpData) >= 2:
                                                styleID = tmpData[0]
                                                if styleID != "":
                                                    cellElement.setAttribute("ss:StyleID", styleID)
                                                data = tmpData[1]
                                            result = data
                                            if result != "":
                                                dataElement = parser.createElement("Data")
                                                dataElement.setAttribute("ss:Type", "String")
                                                dataElement.appendChild(parser.createTextNode(result))                                    
                                                cellElement.appendChild(dataElement)
                                            rowElement.appendChild(cellElement)
                                        table.appendChild(rowElement)
                                        flag2 = True
                                        i += 1                                                                               
                                if table.hasAttribute("ss:ExpandedRowCount"):
                                    table.removeAttribute("ss:ExpandedRowCount")
                            break                    
                    if flag0 and not flag2:
                        j = 0
                        while (j < len(sheetList)):
                            if sheetName == sheetList[j].getAttribute("ss:Name"):
                                sheetList[j].parentNode.removeChild(sheetList[j])
                                del sheetList[j]                                
                                break
                            j += 1                    
                    if not flag1:
                        messageList.append("The output file '%s' does not have a sheet '%s'." % (pFile, sheetName))
                i += 1
    
            fileHandler = open(pFile, "w")
            fileHandler.write(parser.toxml("utf-8"))
            fileHandler.close()
            
            return 0, messageList
             
    except:
        if pAppendFlag:
            return -1, ["Error when appending output to file '%s'." % pFile]
        else:
            return -1, ["Error when writing output to file '%s'." % pFile]
   
    
