package org.xdv.clx.base;

import org.xdv.clx.expr.ClxRule;
import org.xdv.common.XdvErrorLevelSet;
import org.xdv.common.XdvValidationError;

/**
 * CLX validation error information - describes why XML document is not valid. 
 *
 * @author Tomasz Kokoszka
 */
public class ClxValidationError implements XdvValidationError {
    
    /** Error level */
    private final int level;
    
    /** Error message */
    private final String message;
    
    /** CLX rule that generated this error */
    private ClxRule rule = null;
    
    /** XML node that (potentially) caused this error - can be null */
    private Object node = null;
    
    /** 
     * Type of XML document - W3C DOM, JDOM.
     * For CLX it depends only on kind of XPathEngine binded to CLX validator.     
     */
    private Class documentType = null;
    
    /**
     * Creates new validation error with level ERROR.
     * @param message error message
     */
    public ClxValidationError(String message) {
        this.message = message;
        this.level = XdvValidationError.LEVEL_ERROR;
    }
    
    /**
     * Creates new validation error.
     * @param level error level - {@link XdvValidationError#LEVEL_ERROR} or 
     *        {@link XdvValidationError#LEVEL_WARNING}
     * @param message error message
     * @throws IndexOutOfBoundsException invalid <code>level</code> value
     */
    public ClxValidationError(int level, String message) {
        if (! XdvErrorLevelSet.ALL.contains(level)) {
            throw new IndexOutOfBoundsException("Value must be " + XdvValidationError.LEVEL_ERROR + 
                    " or " + XdvValidationError.LEVEL_WARNING + ", not " + level);
        }
        this.message = message;
        this.level = level;
    }
        
    /**
     * Returns type of XML document that was validated - W3C DOM, JDOM, etc.
     * @return type of validated XML document; !=null
     */
    public Class getDocumentType() {
        return documentType;
    }

    /**
     * Sets type of XML document that was validated - W3C DOM, JDOM, etc.
     * @param documentType XML document type, for instance {@link org.w3c.dom.Document}
     */
    public void setDocumentType(Class documentType) {
        this.documentType = documentType;
    }
    
    /**
     * Returns CLX rule that generated this error.
     * @return CLX rule that generated this error; can be null
     */
    public ClxRule getRule() {
        return rule;
    }
    
    /**
     * Sets CLX rule that generated this error.
     * @param rule CLX rule that generated this error
     */
    public void setRule(ClxRule rule) {
        this.rule = rule;
    }

    /**
     * Returns XML node that (potentially) caused this error
     * @return XML node that (potentially) caused this error - can be null
     */
    public Object getNode() {
        return node;
    }

    /**
     * Sets XML node that caused this error.
     * On the node that is <code>!=null</code> is stored, all further calls to this method
     * does nothing. 
     * @param errorNode XML node that cause this error; can be null
     */
    public void addNode(Object errorNode) {
        if (this.node == null) {
            this.node = errorNode;
        }
    }        

    public int getLevel() {
        return level;
    }

    public String getMessage() {
        return message;
    }

    /**
     * Returns error cause - a XML node that caused this error.
     * Type of XML node depends on {@link #getCauseType()}. 
     * @return error cause, can be null
     */
    public Object getCause() {
        return node;
    }

    /**
     * Returns type of error cause - a type of XML document that was validated.
     * For instance it would be {@link org.w3c.dom.Document} for W3C DOM source.
     * Type of XMl document determines type of XML node returned by {@link #getCause()}.  
     * @return type of XML document that was validated; !=null
     */
    public Object getCauseType() {
        return documentType;
    }
       
}
