package org.xdv.clx.extension;

import java.util.Map;

import org.xdv.clx.exec.ClxExecutionContext;
import org.xdv.clx.expr.ClxFormula;
import org.xdv.clx.expr.ClxMacro;
import org.xdv.common.XdvValidationException;

/**
 * Lazy MACRO CALL - CLX extended element implementation.
 * <p>
 * This call looks for macro definition at execution time.
 * It means that it support recurent and forwarding calls, but is slower than other implementations 
 * - every time it searchs macro definition in a map.
 * </p>
 * 
 * @author Tomasz Kokoszka
 */
public class ClxExtMacroCallLazy extends ClxExtMacroCall {
    
    /** Macro name */
    private final String macroName;
    
    /**
     * Create lazy macro call.
     * @param name macro name
     * @param params macro parameters: parameter name-&gt;expression to evaluate its value 
     *        ({@link String}-&gt;{@link org.xdv.xpath.XPathExpression})
     */
    public ClxExtMacroCallLazy(String name, Map params) {
        super(params);
        this.macroName = name;
    }
    
    public String getMacroName() {
        return macroName;
    }

    /**
     * Returns macro output - formula that will executed by this macro call.
     * <br>
     * This implementation looks for macro definition using 
     * {@link ClxExecutionContext#getMacro(String)}. Then it checks parameters for execution and
     * call (those sets have to be equal). That is why this implementation may be pretty slow in
     * comparision to others.    
     * @param cntx execution context
     * @return formula to execute; !=null
     * @throws XdvValidationException unable to find required macro definition or macro parameters
     *         for call and defintiions are not equal
     */
    protected ClxFormula getOutput(ClxExecutionContext cntx) 
    throws XdvValidationException {
        ClxMacro macro = cntx.getMacro(this.macroName);
        if (macro == null) {
            throw new XdvValidationException("Macro " + macroName + " not defined");
        }
        if (! macro.getParams().equals(params.keySet())) {
            throw new XdvValidationException("Parameter list for call and definition " +
                    "doesn't match for macro " + macroName); 
        }
        return macro.getOutput();
    }       
    
    public String toString() {
        return "macroCall:lazy(" + macroName + ")" + super.toString();
    }

}
