package org.xdv.common;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Set of convenient static methods to operate on XML W3C DOM source.
 * <p>
 * This class provides a common functionality to operate on XML W3C DOM source, like:
 * <ul class="NoTopMargin">
 *  <li>Loading XML document from stream or file into W3C DOM object using JAXP.
 *  <li>Geting direct children (namespace aware or not).
 *  <li>Checking namespace.
 *  <li>Getting element/attribute value.
 *  <li>Printing elements.
 *  <li>Formatting element's value, escaping XML entites.
 *  <li>etc.
 * </ul>
 * This class should never be instantiated.
 * </p>
 * 
 * @author Tomasz Kokoszka
 */
public class UtilDOM {

    /**
     * Creates JAXP document builder.
     * Created builder doesn't validate agains XML schema, is namespace aware, optimizes white 
     * spaces and XML comments. 
     * @return JAXP document builder; !=null
     * @throws ParserConfigurationException JAXP is not properly configured 
     */
    public static DocumentBuilder createDocumentFactoryJAXP() 
    throws ParserConfigurationException {    
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setValidating(false);
        factory.setCoalescing(true);        
        factory.setIgnoringElementContentWhitespace(true);
        factory.setExpandEntityReferences(true);
        factory.setIgnoringComments(true);
        factory.setNamespaceAware(true);
        return factory.newDocumentBuilder();
    }
    
    /**
     * Loads XML document from file using JAXP.
     * It creates JAXP document builder using {@link #createDocumentFactoryJAXP()}.
     * @param file file name
     * @return Loaded XML document as W3C DOM object; !=null
     * @throws ParserConfigurationException JAXP is not properly configured
     * @throws SAXException error while parsing the file
     * @throws IOException error while reading the file
     */
    public static Document loadXMLusingJAXP(File file)
    throws ParserConfigurationException, SAXException, IOException {
        DocumentBuilder builder = createDocumentFactoryJAXP();    
        return builder.parse(file);
    }
    
    /**
     * Loads XML document from stream using JAXP.
     * It creates JAXP document builder using {@link #createDocumentFactoryJAXP()}.
     * @param is input stream
     * @return Loaded XML document as W3C DOM object; !=null
     * @throws ParserConfigurationException JAXP is not properly configured
     * @throws SAXException error while parsing the stream
     * @throws IOException error while reading from the stream
     */
    public static Document loadXMLusingJAXP(InputStream is)
    throws ParserConfigurationException, SAXException, IOException {
        DocumentBuilder builder = createDocumentFactoryJAXP();    
        return builder.parse(is);
    }
    
    /**
     * Does given namespace URI points to <i>default/empty</i> namespace?
     * @param namespaceURI namespace URI to be checked; can be null
     * @return True if given namespace URI is <code>null</code> or an empty string
     */
    static public boolean isEmptyNS(String namespaceURI) {
        return (namespaceURI == null) || (namespaceURI.length() == 0); 
    }
    
    /**
     * Does given namespace URI is the same as expected or is empty?
     * @param checkNS namespace URI to check; can be null
     * @param acceptNS expected namespace URI
     * @param acceptEmptyNS true means that we also accept an <i>default/empty</i> namespace
     * @return Value <code>true</code> if:<br> 
     * <code>checkNS=acceptNS or (acceptEmptyNS=true and isEmptyNS(checkNS))</code>
     */
    static public boolean isNS(String checkNS, String acceptNS, boolean acceptEmptyNS) {
        return acceptNS.equals(checkNS) || (acceptEmptyNS && isEmptyNS(checkNS));
    }

    /**
     * Returns simplified path to given node.
     * @param elem XML node
     * @return Simplified XPath go given node; !=null. 
     *         <br>For instance it will return <code>/data/field</code> for XML node 
     *         <code>/data/field[2]</code>
     */
    static public String getSimplePath(Node elem) {
        List names = new ArrayList();
        
        if (elem.getNodeType() == Node.ATTRIBUTE_NODE) {
            Attr a = (Attr) elem;
            names.add("@" + a.getName());
            elem = a.getOwnerElement();
        }
        
        Node e = elem;
        while((e != null) && (e.getNodeType() != Node.DOCUMENT_NODE)) {
            names.add(e.getNodeName());
            e = e.getParentNode();
        }
        
        StringBuffer sb = new StringBuffer("/");
        for(ListIterator i = names.listIterator(names.size()); i.hasPrevious(); ) {
            final String name = (String) i.previous();
            sb.append(name);
            if (i.hasPrevious()) {
                sb.append("/");
            }
        }
        
        return sb.toString();
    }
    
    /**
     * Returns all direct children (from all namespaces).
     * It takes all direct children which are of type ELEMENT and puts them into result list
     * in the same order as they appear in document. 
     * @param root XML element
     * @return List of {@link Element}; !=null
     */
    static public List getChildren(Element root) {
        final NodeList nl = root.getChildNodes();
        final int size = nl.getLength();
        final List result = new ArrayList();
        for(int i = 0; i < size; i++) {
            final Node n = nl.item(i);
            if (n.getNodeType() == Node.ELEMENT_NODE) {
                result.add(n);
            }
        }
        
        return result;
    }
    
    /**
     * Returns all direct children with given name and namespace.
     * It takes direct children that meets conditions given bellow and puts them into result list 
     * in the same order as they appear in document.
     * Child must meet following conditions:
     * <ol>
     *  <li>Is of type ELEMENT
     *  <li>Has local name equal to <code>elemLocalName</code>
     *  <li>Comes from namespace <code>namespaceURI</code> or from <i>empty/default</i> namespace (only
     *      when <code>allowEmptyNS=true</code>)  
     * </ol> 
     * @param root XML element
     * @return List of {@link Element}; !=null
     */
    static public List getChildrenNS(Element root, String elemLocalName, 
            String namespaceURI, boolean allowEmptyNS) {
        final NodeList nl = root.getChildNodes();
        final int size = nl.getLength();
        final List result = new ArrayList();
        for(int i = 0; i < size; i++) {
            final Node n = nl.item(i);
            if (n.getNodeType() == Node.ELEMENT_NODE) {                
                final String ns = n.getNamespaceURI();
                if ((allowEmptyNS && isEmptyNS(ns) && elemLocalName.equals(n.getNodeName())) ||
                    (namespaceURI.equals(ns) && elemLocalName.equals(n.getLocalName()))) {
                    result.add(n);
                }
            }
        }
        
        return result;
    }

        
    /**
     * Returns attribute value.
     * @param root XML element that hold attribute
     * @param attrName attribute name
     * @param defaultValue default value returned when attribute doesn't exist
     * @return Attribute value or <code>defaultValue</code> if attribute doesn't exist
     */
    static public String getAttributeValue(Element root, String attrName, String defaultValue) {
        final Attr a = root.getAttributeNode(attrName);
        if (a == null) {
            return defaultValue;
        }
        return a.getValue();
    }
    
    /**
     * Returns element value - concatenation of all direct text children.
     * @param root element
     * @return Element value; !=null. If element doesn't have value then an empty text is returned.
     */
    public static String getElementValue(Element root) {
        StringBuffer sb = new StringBuffer();
        final NodeList nl = root.getChildNodes();
        final int size = nl.getLength();
        for(int i = 0; i < size; i++) {
            final Node n = nl.item(i);
            if ((n.getNodeType() == Node.TEXT_NODE) ||
                (n.getNodeType() == Node.CDATA_SECTION_NODE)) {
                sb.append(n.getNodeValue());
            }
        }
        return sb.toString().trim();
    }
        
    /**
     * Formats given text using rule: every chain of whitespaces is converted to one space.
     * @param text text to format
     * @param trimLeft true means trim all most left whitespaces from result
     * @param trimRight true means trim all most right whitespaces from result
     * @return Formatted text; !=null
     */
    public static String formatWhitespaces(String text, boolean trimLeft, boolean trimRight) {
        StringBuffer sb = new StringBuffer(text.length());
        
        boolean inWhitespaces = trimLeft;
        for(int i = 0; i < text.length(); i++) {
            char c = text.charAt(i);
            if (Character.isWhitespace(c)) {
                if (! inWhitespaces) {
                    sb.append(' ');
                }                
                inWhitespaces = true;
            } else {
                sb.append(c);
                inWhitespaces = false;
            }
        }
        
        if (trimRight && 
            (sb.length() > 0) && 
            Character.isWhitespace(sb.charAt(sb.length() - 1))) {
            sb.deleteCharAt(sb.length() - 1);
        }
        
        return sb.toString();
    }
    
    /**
     * Converts characters that cannot appear in XML as an element value into equivalent 
     * replacements.
     * It converts:
     * <ul>
     *  <li>'<code>&lt;</code>' to '<code>&amp;lt;</code>'
     *  <li>'<code>&gt;</code>' to '<code>&amp;gt;</code>'
     * </ul>
     * @param text source text
     * @return Converted source text; !=null
     */
    public static String escapeElementText(String text) {
        final StringBuffer sb = new StringBuffer(text.length());
        
        for(int i = 0; i < text.length(); i++) {
            char c = text.charAt(i);
            switch(c) {
            case '<':
                sb.append("&lt;");
                break;
            case '>':
                sb.append("&gt;");
                break;
            default:
                sb.append(c);
            }
        }       
        
        return sb.toString();
    }
    
    /**
     * Converts characters that cannot appear in XML as an attribute value into equivalent 
     * replacements.
     * It converts:
     * <ul>
     *  <li>'<code>&quot;</code>' to '<code>&amp;quot;</code>'
     * </ul>
     * @param text source text
     * @return Converted source text; !=null
     */
    public static String escapeAttributeText(String text) {
        final StringBuffer sb = new StringBuffer(text.length());
        
        for(int i = 0; i < text.length(); i++) {
            char c = text.charAt(i);
            switch(c) {
            case '"':
                sb.append("&quot;");
                break;
            default:
                sb.append(c);
            }
        }       
        
        return sb.toString();
    }
    
    /**
     * Prints whole element content into a new string.
     * For instance, for element defined like this:
     * <br>
     * &nbsp;&nbsp;&nbsp;&nbsp;
     * <code>&lt;element&gt;It is a sample &lt;b&gt;value&lt;/b&gt;&lt;/element&gt;</code>
     * <br>
     * it will generate following text:
     * <br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>It is a sample &lt;b&gt;value&lt;/b&gt;</code> 
     * 
     * @param elem element which content will be printed
     * @return Element content as a text; !=null
     */
    public static String printElementContent(Element elem) {
        final StringBuffer sb = new StringBuffer();
        
        final NodeList nl = elem.getChildNodes();
        final int size = nl.getLength();
        for(int i = 0; i < size; i++) {
            final Node n = nl.item(i);
            switch(n.getNodeType()) {
            case Node.ELEMENT_NODE:
                sb.append(printElement((Element) n));
                break;
            case Node.COMMENT_NODE:
                sb.append("<!--");
                sb.append(escapeElementText(n.getNodeValue()));
                sb.append("-->");
                break;
            case Node.PROCESSING_INSTRUCTION_NODE:
                sb.append("<?").append(n.getNodeName()).append(" ");
                sb.append(escapeElementText(n.getNodeValue()));
                sb.append("?>");
                break;
            default:
                if (n.getNodeValue() != null) {
                    sb.append(escapeElementText(n.getNodeValue()));
                }
            }
                
        }
        
        return sb.toString();
    }
    
    /**
     * Prints whole element definition as a text.
     * For instance, for element defined like this:
     * <br>
     * &nbsp;&nbsp;&nbsp;&nbsp;
     * <code>&lt;element&gt;It is a sample &lt;b&gt;value&lt;/b&gt;&lt;/element&gt;</code>
     * <br>
     * it will generate following text:
     * <br>
     * &nbsp;&nbsp;&nbsp;&nbsp;
     * <code>&lt;element&gt;It is a sample &lt;b&gt;value&lt;/b&gt;&lt;/element&gt;</code> 
     * 
     * @param elem element to print
     * @return Element definition as a text; !=null
     */
    public static String printElement(Element elem) {
        final StringBuffer sb = new StringBuffer();
        sb.append("<").append(elem.getNodeName());
        
        //attributes
        final NamedNodeMap attrs = elem.getAttributes();
        final int size = attrs.getLength();
        for(int i = 0; i < size; i++) {
            final Attr a = (Attr) attrs.item(i);
            sb.append(" ").append(a.getNodeName()).append("=\"");
            sb.append(escapeAttributeText(a.getValue()));
            sb.append("\"");
        }
        
        //content
        String content = printElementContent(elem);
        if (content.length() > 0) {
            sb.append(">").append(content).append("</").append(elem.getNodeName()).append(">");
        } else {
            sb.append("/>");
        }
        
        return sb.toString();
    }
    
    /**
     * Private constructor to prevent creating instances.
     */
    private UtilDOM() {
    }

}
