package org.xdv.common;

/**
 * Set of validation error levels.
 * This set is dedicated to store only error levels - 
 * {@link XdvValidationError#LEVEL_ERROR}, {@link XdvValidationError#LEVEL_WARNING}.
 *
 * @see XdvValidationError
 * @author Tomasz Kokoszka
 */
public class XdvErrorLevelSet {
    
    /** Singleton - one size set, contains only level {@link XdvValidationError#LEVEL_ERROR}. */
    public static final XdvErrorLevelSet ERROR = new XdvErrorLevelSet(true, false);
    
    /** Singleton - one size set, contains only level {@link XdvValidationError#LEVEL_WARNING}. */
    public static final XdvErrorLevelSet WARNING = new XdvErrorLevelSet(false, true);
    
    /** 
     * Singleton - contains all known error levels. 
     */
    public static final XdvErrorLevelSet ALL = new XdvErrorLevelSet(true, true);

    /** Does this set contains level ERROR? */
    private boolean error = false;
    
    /** Does this set contains level WARNING? */
    private boolean warning = false;
    
    /** Creates an empty set */
    public XdvErrorLevelSet() {
    }
    
    /**
     * Creates a set.
     * @param error if true, created set will contain level ERROR 
     *        ({@link XdvValidationError#LEVEL_ERROR})
     * @param warning if true, created set will contain level WARNING 
     *        ({@link XdvValidationError#LEVEL_WARNING})
     */
    public XdvErrorLevelSet(boolean error, boolean warning) {
        this.error = error;
        this.warning = warning;
    }
    
    /**
     * Adds given level to this set.
     * @param level error level, {@link XdvValidationError#LEVEL_ERROR} or 
     *        {@link XdvValidationError#LEVEL_WARNING}
     * @throws IndexOutOfBoundsException invalid <code>level</code> value
     */
    public void add(int level) 
    throws IndexOutOfBoundsException {
        switch(level) {
        case XdvValidationError.LEVEL_ERROR:            
            error = true;
            return;
        case XdvValidationError.LEVEL_WARNING:
            warning = true;
            return;
        default:
            throw new IndexOutOfBoundsException("Value must be " + XdvValidationError.LEVEL_ERROR + 
                    " or " + XdvValidationError.LEVEL_WARNING + ", not " + level);
        }
    }
    
    /**
     * Checks if set contains given level.
     * @param level error level
     * @return true only if set contains given level
     */
    public boolean contains(int level) {
        if (level == XdvValidationError.LEVEL_ERROR) {
            return error;
        }
        if (level == XdvValidationError.LEVEL_WARNING) {
            return warning;
        }
        return false;
    }
            
}
