package org.xdv.xpath;

/**
 * XPath value.
 * <p>
 * It differs four XPath 1.0 data types:
 * <ol class="NoTopMargin">
 * <li><a href="http://www.w3.org/TR/xpath#strings">String<a/> - {@link #TYPE_STRING},
 * <li><a href="http://www.w3.org/TR/xpath#numbers">Number</a> (decimal) - {@link #TYPE_NUMBER},
 * <li><a href="http://www.w3.org/TR/xpath#booleans">Boolean</a> (true/false) - {@link #TYPE_BOOLEAN},
 * <li><a href="http://www.w3.org/TR/xpath#node-sets">Node set</a> - {@link #TYPE_NODESET}.
 * </ol>
 * Single XML node should be stored as a node-set containing exactly one node ({@link #TYPE_NODESET}).
 * </p>
 * <p>
 * Instance should never be created directly but through {@link XPathEngine}:
 * <ul class="NoTopMargin">
 * <li>{@link XPathEngine#createValue(Object)} - type of created value depends on object type and
 *     underlying XPath implementation
 * <li>{@link XPathEngine#createValueString(String)} - creates a string value,
 * <li>{@link XPathEngine#createValueNumber(double)} - creates a number value,
 * <li>{@link XPathEngine#createValueBoolean(boolean)} - creates a boolean value,
 * <li>{@link XPathEngine#createValueNodeSet(java.util.Collection)} - creates a node-set,
 * <li>{@link XPathEngine#createValueNode(Object)} - creates a node-set containing exactly one XML node.
 * </ul>
 * </p>
 * <p>
 * XPath values are immutable in sense that you cannot change value that they wrap - 
 * once created they always wrap the same object.
 * Same as Java primitives wrappers ({@link String}, {@link Integer} etc.).
 * </p>
 *
 * @author Tomasz Kokoszka
 */
public interface XPathValue {
    
    /** XPath 1.0 type identifier - unknown type */
    public static final int TYPE_OTHER = 0;
    
    /** XPath 1.0 type identifier - string */
    public static final int TYPE_STRING = 1;
    
    /** XPath 1.0 type identifier - number (decimal, IEEE754) */
    public static final int TYPE_NUMBER = 2;
    
    /** XPath 1.0 type identifier - boolean (true/false) */
    public static final int TYPE_BOOLEAN = 3;
    
    /** XPath 1.0 type identifier - node set */
    public static final int TYPE_NODESET = 4;
            
    /**
     * Converts value to a string.
     * <p>
     * Conversion rules:
     * <ol class="NoTopMargin">
     * <li>Type {@link #TYPE_STRING} - wrapped text
     * <li>Type {@link #TYPE_NUMBER} - wrapped number as a text (IEEE754), 
     *     <code>NaN</code> is converted to 'NaN', <code>0.0</code> to '0', 5.23400 to '5.234' etc.
     * <li>Type {@link #TYPE_BOOLEAN} - <code>true</code> to 'true', <code>false</code> to 'false'
     * <li>Type {@link #TYPE_NODESET} - concatenated string value of each node in the set 
     * <li>Type {@link #TYPE_OTHER} - not defined
     * <li>XML node in {@link #TYPE_NODESET} - depending of node type:
     *     <ol class="NoTopMargin" type="a">
     *     <li>element - concatenated value of all text nodes children of the element,
     *     <li>attribute - value of attribute,
     *     <li>text node - content of text node,
     *     <li>comment - content of the comment, excluding the <code>&lt;!--</code> and
     *         <code>--&gt;</code> escape characters
     *     <li>processing instruction - content of the processing instruction following the target,
     *         but excluding the terminating <code>?&gt;</code>
     *     <li>other - not defined
     *     </ol>      
     * </ol>
     * </p>  
     * 
     * @return Value as text; !=null
     */
    public String asString();
    
    /**
     * Converts value to a number.
     * <p>
     * Conversion rules:
     * <ol class="NoTopMargin">
     * <li>Type {@link #TYPE_NUMBER} - wrapped number
     * <li>Type {@link #TYPE_BOOLEAN} - <code>true</code> to '1', <code>false</code> to '0'
     * <li>All other types - take {@link #asString()} and convert returned value to 
     *     a decimal number. <code>NaN</code> is returned if conversion cannot be performed.
     * </ol>
     * </p> 
     * 
     * @return Value as a number 
     */
    public double asNumber();
    
    /**
     * Converts value to a boolean value.
     * <p>
     * Conversion rules:
     * <ol class="NoTopMargin">
     * <li>Type {@link #TYPE_BOOLEAN} - wrapped value
     * <li>Type {@link #TYPE_NUMBER} - non-zero number except <code>NaN</code> is converted to 
     *     <code>true</code>, <code>0</code> and <code>NaN</code> are converted to <code>false</code>
     * <li>Type {@link #TYPE_NODESET} - non-empty set is converted to <code>true</code>, empty
     *     set to <code>false</code>
     * <li>All other types - take {@link #asString()}, if value is equal (case insensitive)
     *     to 'true' returns <code>true</code>, otherwise returns <code>false</code>  
     * </ol>
     * </p>
     * @return Value as a boolean value 
     */ 
    public boolean asBoolean();
    
    /**
     * Returns native object used by underlying implementation to represent such value.
     * @return Wrapped object, object type depends on underlying XPath implementation
     */
    public Object getNative();
    
    /**
     * Returns value type.
     * Type it is one of {@link #TYPE_STRING}, {@link #TYPE_NUMBER},
     * {@link #TYPE_BOOLEAN}, {@link #TYPE_NODESET},
     * {@link #TYPE_OTHER}.
     * @return Value type
     */
    public int getType();
    
}
