package org.xdv.xpath.common;

import org.xdv.xpath.XPathValue;

/**
 * XPath numeric value - wraps {@link java.lang.Double}.
 * It is a simple wrapper for {@link Double}. 
 * Can be used by any XPath implementation that maps {@link Double} to XPath 1.0 numeric type. 
 * <p>
 * Instance should never be created directly but rather using suitable XPath engine.
 * Instances are immutable - it means that its value cannot be changed.
 * </p>
 * 
 * @author Tomasz Kokoszka
 */
public class BaseXPathValueNumber implements XPathValue {

    /** Wrapped value - immutable. !=null */
    private final Double value; 
    
    /**
     * Creates wrapper for given {@link Double} object.
     * @param value object to wrap
     */
    public BaseXPathValueNumber(Double value) {
        assert value != null;
        this.value = value;
    }
    
    /**
     * Returns wrapped object.
     * @return Wrapped object, !=null
     */
    public Double getValue() {
        return this.value;
    }

    /**
     * Returns wrapped object - instance of {@link Double}.
     * @return Wrapped object - {@link Double}, !=null
     */
    public Object getNative() {
        return value;
    }

    /**
     * Returns value type.
     * @return {@link XPathValue#TYPE_NUMBER}
     */
    public int getType() {
        return XPathValue.TYPE_NUMBER;
    }
    
    public String asString() {
        return value.toString();
    }

    public boolean asBoolean() {
        return (! Double.isNaN(value.doubleValue())) && (value.doubleValue() != 0);
    }

    public double asNumber() {
        return value.doubleValue();
    }
    
    public String toString() {
        return value.toString();
    }
       
}
