package org.xdv.xpath.common;

import org.xdv.xpath.XPathValue;

/**
 * XPath text value - wraps {@link java.lang.String}.
 * It is a simple wrapper for {@link String}. 
 * Can be used by any XPath implementation that maps {@link String} to XPath 1.0 string type. 
 * <p>
 * Instance should never be created directly but rather using suitable XPath engine.
 * Instances are immutable - it means that its value cannot be changed.
 * </p>
 * 
 * @see XPathValue
 * @author Tomasz Kokoszka
 */
public class BaseXPathValueString implements XPathValue {
   
    /** Wrapped value - immutable. !=null */
    private final String value;
    
    /**
     * Creates wrapper for given {@link String} object.
     * @param value object to wrap
     */
    public BaseXPathValueString(String value) {
        assert value != null;
        this.value = value;
    }
    
    /**
     * Returns wrapped object.
     * @return Wrapped object, !=null
     */
    public String getValue() {
        return this.value;
    }

    /**
     * Returns wrapped object - instance of {@link String}.
     * @return Wrapped object - {@link String}, !=null
     */
    public Object getNative() {
        return value;
    }

    /**
     * Returns value type.
     * @return {@link XPathValue#TYPE_STRING}
     */
    public int getType() {
        return XPathValue.TYPE_STRING;
    }
    
    public String asString() {
        return value;
    }

    public boolean asBoolean() {
        return Boolean.valueOf(value).booleanValue();
    }

    public double asNumber() {
        try {
            return Double.parseDouble(value);
        } catch (NumberFormatException e) {
            return Double.NaN;
        }
    }
    
    public String toString() {
        return value;
    }
    
}
