package org.xdv.xpath.jaxen;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.jaxen.JaxenException;
import org.jaxen.dom.DOMXPath;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xdv.xpath.XPathEngine;
import org.xdv.xpath.XPathException;
import org.xdv.xpath.XPathExecutionUnit;
import org.xdv.xpath.XPathExpression;
import org.xdv.xpath.XPathInvalidTypeError;
import org.xdv.xpath.XPathValue;
import org.xdv.xpath.XPathValueNodeSet;
import org.xdv.xpath.XPathVariableMap;
import org.xdv.xpath.common.BaseXPathValueBoolean;
import org.xdv.xpath.common.BaseXPathValueNumber;
import org.xdv.xpath.common.BaseXPathValueString;
import org.xdv.xpath.common.DOMXPathValueNode;

/**
 * Jaxen XPath engine for W3C DOM source.
 * <br>
 * Works with following type mapping (XPath 1.0 type -&gt; {@link XPathValue}):
 * <ol style="NoTopMargin">
 *  <li>XPath 1.0 string -&gt; {@link org.xdv.xpath.common.BaseXPathValueString}
 *  <li>XPath 1.0 number -&gt; {@link org.xdv.xpath.common.BaseXPathValueNumber}
 *  <li>XPath 1.0 boolean -&gt; {@link org.xdv.xpath.common.BaseXPathValueBoolean}
 *  <li>XPath 1.0 node-set -&gt; {@link org.xdv.xpath.jaxen.JaxenXPathValueNodeSet}
 *  <li>single node in node-set -&gt; {@link org.xdv.xpath.common.DOMXPathValueNode}
 * </ol>
 * </p>
 * <p>
 * Objects created by this engine will not work with object created by other XPath API engine.
 * <br> 
 * Engine instances don't have state, so they can be used interchangeably.
 * </p>
 * 
 * @see org.xdv.xpath.jaxen
 * @author Tomasz Kokoszka
 */
public class JaxenXPathEngine implements XPathEngine {
    
    /** 
     * Creates new wrapper for Jaxen W3C DOM XPath 1.0 engine. 
     */
    public JaxenXPathEngine() {
    }
    
    /**
     * Returns accepted source XML document class - W3C DOM {@link Document}.
     * @return Class {@link Document}; !=null
     */
    public Class getDocumentClass() {
        return Document.class;
    }   
    
    /**
     * Returns accepted source XML node class - W3C DOM {@link Node}.
     * @return Class {@link Document}; !=null
     */
    public Class getNodeClass() {
        return Node.class;
    }
    
    public String getNodeString(Object nodeNative)
    throws XPathInvalidTypeError {
        if (! (nodeNative instanceof Node)) {
            throw new XPathInvalidTypeError(nodeNative, this.getNodeClass());
        }
        return DOMXPathValueNode.toString((Node) nodeNative);
    }

    public boolean isSameNode(Object nodeNative1, Object nodeNative2) {
        return nodeNative1.equals(nodeNative2);
    }

    /**
     * Creates compiled XPath expression.
     * @param xpath XPath expression as text, for instance: 
     *        <code>/data/field[substring(@name, 1, 2)='In']</code>
     * @return Wrapped XPath expression; !=null. Instance of {@link JaxenXPathExpression} 
     * @throws XPathException given text is not valid XPath 1.0 expression
     */
    public XPathExpression createExpression(String xpath) 
    throws XPathException {
        DOMXPath x;
        try {        
            x = new DOMXPath(xpath);
        } catch (JaxenException e) {
            throw new XPathException(e.getMessage(), xpath, e);
        }        
        
        final JaxenXPathExpression expression = new JaxenXPathExpression(x);        
        return expression;
    }

    /**
     * Creates XPath execution unit.
     * @param documentNative source XML document. It must be an instance of {@link Document}, 
     *        otherwise a {@link XPathInvalidTypeError} is thrown. 
     * @return Execution unit; !=null. Instance of {@link JaxenXPathExecutionUnit}.
     * @throws XPathInvalidTypeError invalid type of XML document. It must be an instance of 
     *         {@link Document}.
     */
    public XPathExecutionUnit createExecutionUnit(Object documentNative)
    throws XPathInvalidTypeError {
        if (! (documentNative instanceof Document)) {
            throw new XPathInvalidTypeError(documentNative, this.getDocumentClass());
        }
        
        return new JaxenXPathExecutionUnit(this, (Document) documentNative);
    }
    
    /**
     * Creates an empty variable map.
     * @return Empty variable map; !=null. Instance of {@link JaxenXPathVariableMap}. 
     */
    public XPathVariableMap createVariableMap() {
        return new JaxenXPathVariableMap();
    }
    
    /**
     * Creates an empty variable map.
     * @return Empty variable map; !=null 
     */
    public JaxenXPathVariableMap createJaxenVariableMap() {
        return new JaxenXPathVariableMap();
    }
    
    /**
     * Creates wrapper for a general native value.
     * <p>
     * This implementation accepts following object as an input:
     * <ol class="NoTopMargin">
     * <li>{@link List} of {@link Node} - produces {@link JaxenXPathValueNodeSet} (XPath node-set)
     * <li>{@link Node} - produces {@link DOMXPathValueNode} (single node-set)
     * <li>{@link String} - produces {@link BaseXPathValueString} (XPath 1.0 string)
     * <li>{@link Double} - produces {@link BaseXPathValueNumber} (XPath 1.0 number)
     * <li>{@link Boolean} - produces {@link BaseXPathValueBoolean} (XPath 1.0 boolean)
     * </ol>  
     * </p>
     * @param valueNative value to wrap into XPath value. See list of accepted classes above.
     * @return Wrapper for native value; !=null. See conversion rules above.
     * @throws XPathException unable to create wrapper, source object type is not on the list of
     *         accepted classes - see list of accepted classes above
     */
    public XPathValue createValue(Object valueNative) throws XPathException {
        if (valueNative instanceof List) {
            final List l = (List) valueNative;
            if (l.size() == 1) {
                return new DOMXPathValueNode((Node) l.get(0));
            }
            return new JaxenXPathValueNodeSet(l);
        } else if (valueNative instanceof Node) {
            return new DOMXPathValueNode((Node) valueNative);
        } else if (valueNative instanceof String) {
            return new BaseXPathValueString((String) valueNative);
        } else if (valueNative instanceof Boolean) {
            return new BaseXPathValueBoolean((Boolean) valueNative);
        } else if (valueNative instanceof Double) {
            return new BaseXPathValueNumber((Double) valueNative);
        }
        
        throw new XPathException("Unknown class " + valueNative.getClass().getName());
    }

    /**
     * Creates wrapper for boolean value.
     * @param value boolean value         
     * @return Instance of {@link BaseXPathValueBoolean}; !=null
     */
    public XPathValue createValueBoolean(boolean value) {
        return new BaseXPathValueBoolean(new Boolean(value));
    }

    /**
     * Creates wrapper for XML node.
     * @param nodeNative W3C node. It must be an instance {@link Node}, otherwise a 
     *        {@link XPathInvalidTypeError} is thrown         
     * @return Instance of {@link DOMXPathValueNode}; !=null
     * @throws XPathInvalidTypeError source object is not an instance of {@link Node}
     */
    public XPathValueNodeSet createValueNode(Object nodeNative)
    throws XPathInvalidTypeError {
        if (! (nodeNative instanceof Node)) {
            throw new XPathInvalidTypeError(nodeNative, this.getNodeClass());
        }
        
        return new DOMXPathValueNode((Node) nodeNative);
    }
    
    /**
     * Creates wrapper for node-set.
     * @param nodesNative collection of {@link Node}, 
     *        otherwise a {@link XPathInvalidTypeError} is thrown         
     * @return Instance of {@link JaxenXPathValueNodeSet}; !=null
     * @throws XPathInvalidTypeError source object is not an instance of {@link Node}
     */
    public XPathValueNodeSet createValueNodeSet(Collection nodesNative) 
    throws XPathInvalidTypeError {
        final List list = new ArrayList(nodesNative);
        
        if (! list.isEmpty()) {
            final Object node = list.get(0);
            if (! (node instanceof Node)) {
                throw new XPathInvalidTypeError(node, this.getNodeClass());
            } 
        }
        
        return new JaxenXPathValueNodeSet(list);
    }

    /**
     * Creates wrapper for given number.
     * @param value number 
     * @return Instance of {@link BaseXPathValueNumber}; !=null
     */
    public XPathValue createValueNumber(double value) {
        return new BaseXPathValueNumber(new Double(value));
    }

    /**
     * Creates wrapper for given text.
     * @param value text 
     * @return Instance of {@link BaseXPathValueString}; !=null
     */
    public XPathValue createValueString(String value) {
        return new BaseXPathValueString(value);
    }
        
    /**
     * Returns hash code - instances of this class doesn't have a state, so all of them are equal.  
     * @return value <code>0</code>
     */
    public int hashCode() {
        return 0;
    }
        
    /**
     * Compares two engines - instances of this class doesn't have a state, so all of them are equal.  
     * @return true if given object is an instance of {@link JaxenXPathEngine}
     */
    public boolean equals(Object object) {
        if (object instanceof JaxenXPathEngine) {
            return true;
        }
        return false;
    }
        
}
